﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;


using Extensions.Collections;       //RCB4を制御するとき使うので、設定しておく
using Rcb4;                         //



namespace AutoMove
{
    public partial class AutoMove : Form
    {
        public AutoMove()
        {
            InitializeComponent();
            centerPoint = new PointF(pictureBox1.Width / 2, pictureBox1.Height / 2);
           // g = pictureBox1.CreateGraphics();      //ピクチャーボックス内に線が引けるようにする


            pictureBox1.Image = new Bitmap(pictureBox1.Width, pictureBox1.Height);
            g = Graphics.FromImage(pictureBox1.Image);

            DrawNewImage();

            robotXY = new PointF(0, 0);
            robotAngle = 0;
        
        }

        private void DrawNewImage()
        {

            g.Clear(pictureBox1.BackColor);

            pictureBox1.Invalidate();

            Pen dotPen = new Pen(Color.Yellow, 2);
            dotPen.DashStyle = System.Drawing.Drawing2D.DashStyle.Dot;

            g.DrawLine(dotPen, centerPoint.X, Top, centerPoint.X, Bottom);
            g.DrawLine(dotPen, Left, centerPoint.Y, Right, centerPoint.Y);


            g.DrawRectangle(new Pen(Color.Black, 2), 0, 0, pictureBox1.Width, pictureBox1.Height);

            pictureBox1.Invalidate();
        }

        #region 各種パラメータ

        //前進１モーションの距離
        private const double forwardOneSize = 6.6;      
        //回転１モーションの角度
        private const double turnOneSize = 23.0;        


        //ロボット上の距離センサの位置座標（Ｙ軸のみ、Ｘ軸は中心なので０）
        private const double rangePoint = 9.0;          

        //MAPについてのパラメータ設定
        private PointF centerPoint;                     
        //点や線を表示させるためのグラフィック変数
        private Graphics g;                            

        //ピクチャボックスの横が０度なので、縦（９０度）に変更分の値
        private const int defaultAngle = 90;            

        //ロボットのワールド（絶対）座標系
        private PointF robotXY;                        
        //ロボットの絶対角度
        private double robotAngle;                      

        //頭についているサーボのＩＤ
        private const int servoHeadSIO = 1;             
        private const int servoHeadID = 0;

        //SYSTEMデータが必要なので、宣言しておく
        private Config config = new Config();

        //ストップを知らせるためのフラグ
        private bool stopFlag = false;           
        
        #endregion
        
        #region 初期設定
        

        ///コンフィグデータを読み込む関数
        private void GetConfigData()
        {
            ByteList cmd = new ByteList();
            byte[] rx = new byte[5];
            //コマンドの生成
            cmd.Bytes = Command.MoveRamToCom(Rcb4.Rcb4.ConfigRamAddress, 2);
            if (Command.Synchronize(serialPort1, cmd.Bytes, ref rx) == true)
            {
                config[0] = rx[2];
                config[1] = rx[3];
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            try
            {
                serialPort1.Open();             //COMポートを開く
            }
            catch
            {

                MessageBox.Show("COMが開けません！！");
                //  return;                         //失敗したらエラーを出して終わる
                serialPort1.Close();

                Application.Exit();
            }

            //COMが開くことができたらconfigを呼んでおく
            GetConfigData();

        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            serialPort1.Close();
        }

        #endregion      

        #region ＭＡＰに線を引く
        private void PictDrawLine(PointF startPoint, float angle, float range)
        {
     
            int pointSize = 2;

            PointF endPoint = new PointF();

            startPoint.X += centerPoint.X;      //０点（中心）に合わせる
            startPoint.Y += centerPoint.Y;


            float angleRad = (angle - defaultAngle) / 180 * (float)Math.PI;

            endPoint.X = range * (float)Math.Cos(angleRad) + startPoint.X;
            endPoint.Y = range * (float)Math.Sin(angleRad) + startPoint.Y;

            g.DrawLine(new Pen(Color.WhiteSmoke, 2), startPoint, endPoint);
            //距離の点を表示
            g.FillEllipse(Brushes.Brown, endPoint.X - (pointSize / 2), endPoint.Y - (pointSize / 2)
                            , pointSize, pointSize);
            //中心座標を表示
            g.FillEllipse(Brushes.ForestGreen, startPoint.X - (pointSize / 2), startPoint.Y - (pointSize / 2)
                  , pointSize, pointSize);

            pictureBox1.Invalidate();

        }

        #endregion

        #region ＭＡＰに点を打つ
        private void PictDrawPoint(PointF point,Color pointColor)
        {
           
            int pointSize = 2;

            point.X += centerPoint.X;
            point.Y += centerPoint.Y;


            SolidBrush brush = new SolidBrush(pointColor);  
  
            //中心座標を表示
            g.FillEllipse(brush, point.X - (pointSize / 2), point.Y - (pointSize / 2)
                  , pointSize, pointSize);

            pictureBox1.Invalidate();
        }


        private void RangePointDraw(double rangeData, double rangeAngle)
        {
            if (rangeData < 0 || 80 < rangeData) return;              //距離データが0以下の時はデータでないので表示しない

            PointF rangeSensorWorldPoint = new PointF();
            double rangeAngleRad = robotAngle / 180 * Math.PI;
            rangeSensorWorldPoint.X = (float)(rangePoint * Math.Cos(rangeAngleRad)) + robotXY.X;      //距離センサのワールド座標
            rangeSensorWorldPoint.Y = (float)(rangePoint * Math.Sin(rangeAngleRad)) + robotXY.Y;

            double rangeWorldAngleRad = (robotAngle + rangeAngle) / 180 * Math.PI;        //距離センサの向きのワールド座標系

            PointF rangeDataWorldPoint = new PointF();
            rangeDataWorldPoint.X = (float)(rangeData * Math.Cos(rangeWorldAngleRad)) + rangeSensorWorldPoint.X;
            rangeDataWorldPoint.Y = (float)(rangeData * Math.Sin(rangeWorldAngleRad)) + rangeSensorWorldPoint.Y;


            Color pointColor = new Color();
            if (rangeData > 45) pointColor = Color.LightGray;       //45cm以上あるときは正確な値でないので、点を灰色にする
            else pointColor = Color.Black;

            PictDrawPoint(rangeDataWorldPoint, pointColor);

        }



        #endregion

        #region 自作sleep関数（普通のsleepは処理が止まるように見える）

        private void MySleep(int ms)
        {
            System.Diagnostics.Stopwatch sw = new System.Diagnostics.Stopwatch();

            sw.Start();

            ////
            ////    計測中はイベントを発生しているかどうか見に行く
            ////

            for (; ; )
            {
                if (sw.ElapsedMilliseconds > ms)
                {
                    break;
                }
                Application.DoEvents();

            }
            sw.Stop();
            sw.Reset();

        }

        #endregion

        #region モーション再生用の関数集


        /// <summary>
        /// モーションを一時停止させるコマンドの発行
        /// </summary>
        private void MotionSuspend()
        {
            ByteList cmd = new ByteList();
            Byte[] rx = new Byte[4];       //読み取るために必要だが、特に使わない

            //コマンドの生成
            cmd.Bytes = Rcb4.Rcb4.Suspend(config);

            //コマンドの送信
            Command.Synchronize(serialPort1, cmd.Bytes, ref rx);

        }
        /// <summary>
        /// プログラムカウンタの
        /// </summary>
        private void PgCounterReset()
        {
            ByteList cmd = new ByteList();
            Byte[] rx = new Byte[4];       //読み取るために必要だが、特に使わない

            //コマンドの生成
            cmd.Bytes = Rcb4.Rcb4.ResetProgramCounter(Rcb4.Rcb4.MainLoopCmd);

            //コマンドの送信
            Command.Synchronize(serialPort1, cmd.Bytes, ref rx);
        }

        private void MotionAddressSet(int motionNum)
        {
            ByteList cmd = new ByteList();
            Byte[] rx = new Byte[4];       //読み取るために必要だが、特に使わない

            //モーションのアドレスは＝モーションの先頭アドレス＋モーション番号＊１モーション１つあたりの容量
            int address = (motionNum - 1) * Rcb4.Rcb4.MotionSingleDataCount + Rcb4.Rcb4.MotionRomAddress;

            //コマンドの生成
            cmd.Bytes = Rcb4.Rcb4.Play(address);

            //コマンドの送信
            Command.Synchronize(serialPort1, cmd.Bytes, ref rx);
        }

        private void MotionRestart()
        {
            ByteList cmd = new ByteList();
            Byte[] rx = new Byte[4];       //読み取るために必要だが、特に使わない

            //コマンドの生成
            cmd.Bytes = Rcb4.Rcb4.Resume(config);

            //コマンドの送信
            Command.Synchronize(serialPort1, cmd.Bytes, ref rx);
        }


        private void MotionPlay(int motionNum, int waitTime)
        {
            //モーションを一時中断する（サスペンド）
            MotionSuspend();
            //プログラムカウンタとＥＥＰＲＯＭ更新フラグリセット(プログラムカウンタをリセット）
            PgCounterReset();
            //コール命令で移動先アドレスをセット
            MotionAddressSet(motionNum);
            //モーションをリスタート
            MotionRestart();

            MySleep(waitTime);    //モーションが終わるまで待つ
        }

        #endregion

        #region センサデータの取得


        /// <summary>
        /// 距離センサデータを取得する
        /// </summary>
        /// <returns></returns>
        private int GetRangeADData()
        {

            int getSensorNum = 10;              //平均とる回数

            long dataSum = 0;                   //データを足すためのバッファ

            ByteList cmd = new ByteList();      //コマンド用にリストを確保

            Byte[] rx = new Byte[5];            //ＡＤの受け取りは5byte

            cmd.Bytes = Rcb4.Rcb4.AdRead(1);    //AD1につながっているデータを取りに行くコマンド生成

            for (int i = 0; i < getSensorNum; i++)
            {

                if (Command.Synchronize(serialPort1, cmd.Bytes, ref rx) == false)   //コマンドを送って受信する
                {
                    //失敗した場合は、
                    MessageBox.Show("読み取りにいけませんでした");
                    return -1;

                }
                else                                                                //成功した場合
                {
                    byte[] ADData = { rx[2], rx[3] };           //AD値を一度byte配列に代入
                    int data = BitConverter.ToInt16(ADData, 0);        //int型に変換（正確には2byteなのでshout型）
                    //    textBox1.Text = data.ToString();

                    dataSum += data;

                }
            }

            // MessageBox.Show(getSensorNum.ToString() + "回の取得が終わりました");
            //平均値を返す
            return (int)(dataSum / getSensorNum);


        }


        private Double RangeADToConvert(int ADData)
        {

            //エクセルで出した式を用いる
            double distanceData = 0.0002 * (double)ADData - 0.0048;

            //距離は逆数で計算されるので、元に戻す
            distanceData = 1 / distanceData;


            if (distanceData < 10 || 80 < distanceData)
            {
                distanceData = -1;
            }
            return distanceData;

        }

        #endregion

        #region 頭サーボの回転

        /// <summary>
        /// サーボを１つだけ動かすための関数
        /// </summary>
        /// <param name="sioNum"></param>
        /// <param name="sercoID"></param>
        /// <param name="servoPos"></param>
        private void OneServoMove(int sioNum, int servoID, int frameSize, int servoPos)
        {

            ByteList cmd = new ByteList();

            Byte[] rx = new Byte[4];

        //    textBox2.Text = trackBar1.Value.ToString();

            byte sendID = (byte)(servoID * 2 + (sioNum - 1));

            cmd.Bytes = Command.RunSingleServo(sendID, (byte)frameSize, servoPos);

            Command.Synchronize(serialPort1, cmd.Bytes, ref rx);      //コマンドを送る
        }

        /// <summary>
        /// 角度をサーボの設定値にする関数
        /// </summary>
        /// <param name="angle">角度</param>
        /// <returns>サーボの設定値</returns>
        private int servoAngleConvert(double angle)
        {
            int neutral = 7500;       //7500のとき0deg

            int servoAngle = neutral + (int)((angle / 135.0) * 4000);//±135deg =>±4000 


            //サーボ最大角、最小角の設定
            if (servoAngle < 3500) servoAngle = 3500;       //3500より小さいときは3500にする
            if (servoAngle > 11500) servoAngle = 11500;     //11500より大きいときは11500にする


            return servoAngle;
        }


        #endregion

        #region センサ首振り取得、表示関数

        private void RangeDataScan(int scanNum, double angle)
        {
            if (scanNum <= 0) return;           //スキャンする数が0以下（しない）場合は抜ける

            double oneAngleSize = angle / (double)(scanNum-1);      //１回の首振り角度の計算
            double startAngle =  -angle / 2;                 //スタート位置は-側から

            for (int i = 0; i < scanNum; i++)
            {
                double setAngle = startAngle + (i * oneAngleSize);          //首振り角度の計算
                int setServoAngle = servoAngleConvert(setAngle);            //角度をサーボの値へ変換
                OneServoMove(servoHeadSIO, servoHeadID, 1, setServoAngle);  //首のサーボを動かす
                                    
           //     System.Threading.Thread.Sleep(500);

                int rangeADData = GetRangeADData();                         //センサのデータを読み取る
                double rangeData = RangeADToConvert(rangeADData);           //センサのデータを距離データに直す
                RangePointDraw(rangeData, setAngle);                        //距離データをプロット
 
            }
            OneServoMove(servoHeadSIO, servoHeadID, 1, servoAngleConvert(0));          //最後は正面を向く

        }

        private void RangeDataScan2(double[] rangeDataList, double angle)
        {
            int scanNum = rangeDataList.Length;

            if (scanNum <= 0) return;           //スキャンする数が0以下（しない）場合は抜ける

            double oneAngleSize = angle / (double)(scanNum - 1);      //１回の首振り角度の計算
            double startAngle = -angle / 2;                 //スタート位置は-側から

            for (int i = 0; i < scanNum; i++)
            {
                double setAngle = startAngle + (i * oneAngleSize);          //首振り角度の計算
                int setServoAngle = servoAngleConvert(setAngle);            //角度をサーボの値へ変換
                OneServoMove(servoHeadSIO, servoHeadID, 1, setServoAngle);  //首のサーボを動かす
                int rangeADData = GetRangeADData();                         //センサのデータを読み取る
                double rangeData = RangeADToConvert(rangeADData);           //センサのデータを距離データに直す
                rangeDataList[i] = rangeData;                               //配列に入れておく
                RangePointDraw(rangeData, setAngle);                        //距離データをプロット

            }
            OneServoMove(servoHeadSIO, servoHeadID, 1, servoAngleConvert(0));          //最後は正面を向く

        }

        #endregion

        #region ロボットの動き

        private void MoveForward()
        {
            MotionPlay(13, 4000);
            PointF oneSizePoint = new PointF();         //１回あたりのすすむ距離
            double angleRad = robotAngle / 180 * Math.PI;
            oneSizePoint.X = (float)(forwardOneSize * Math.Cos(angleRad));  //X軸移動量の計算
            oneSizePoint.Y = (float)(forwardOneSize * Math.Sin(angleRad));  //Y軸移動量の計算
            robotXY.X += oneSizePoint.X;
            robotXY.Y += oneSizePoint.Y;
            PictDrawPoint(robotXY, Color.Blue);

            roboPosition.Text = "X:" + robotXY.X.ToString("00") + " Y:" + robotXY.Y.ToString("00");
            roboPosition.Invalidate();
        }

        private void TurnRight()
        {
            MotionPlay(5, 500);
            robotAngle += turnOneSize;
            roboAngleLabel.Text = robotAngle.ToString() + "deg";
            roboPosition.Invalidate();
        }

        private void TurnLeft()
        {
            MotionPlay(4, 500);
            robotAngle -= turnOneSize;
            roboAngleLabel.Text = robotAngle.ToString() + "deg";
            roboPosition.Invalidate();
        }


        #endregion


        #region 障害物停止ボタンを押したとき

        /// <summary>
        /// 障害物を見つけて止まるボタンを押した時のイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button1_Click(object sender, EventArgs e)
        {
           
            MotionPlay(1, 1000);

            for (int i= 0;i<1000 ;i++ )         //ずっとループを続ける可能性があるので、１０００回くらいでとめてやる
            {
                int rangeDataI =  GetRangeADData();
                double rangeDataF =  RangeADToConvert(rangeDataI);     //距離データを変換

                if (0 < rangeDataF && rangeDataF <= 30) //距離が３０cm以内の時
                {
                    MotionPlay(1, 1000);
                    break;
                }

                else                            //そうでないときは前進
                {
                   MotionPlay(13, 4000);
                }
            } 

        }

        #endregion
   
        #region １方向回避が押されたとき

        /// <summary>
        /// １方向回避が押されたとき
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button2_Click(object sender, EventArgs e)
        {
            MotionPlay(1, 1000);

            for (int i = 0; i < 1000; i++)         //ずっとループを続ける可能性があるので、１０００回くらいでとめてやる
            {

                if (stopFlag == true)      //ストップボタンが押されたとき
                {
                    stopFlag = false;
                    MotionPlay(1, 500);
                    break;
                }

                int rangeDataI = GetRangeADData();
                double rangeDataF = RangeADToConvert(rangeDataI);     //距離データを変換

                if (0 < rangeDataF && rangeDataF <= 30) //距離が３０cm以内の時
                {
                    MotionPlay(4, 500);
                    MotionPlay(4, 500);
                }

                else                            //そうでないときは前進
                {
                    MotionPlay(13, 4000);
                }

            }
        }

        #endregion

        #region ランダム方向転換ボタンを押したとき

        private void button4_Click(object sender, EventArgs e)
        {
            Graphics g = pictureBox1.CreateGraphics();      //ピクチャーボックス内に線が引けるようにする


            MotionPlay(1, 1000);

            for (int i = 0; i < 1000; i++)         //ずっとループを続ける可能性があるので、１０００回くらいでとめてやる
            {

                if (stopFlag == true)      //ストップボタンが押されたとき
                {
                    stopFlag = false;
                    MotionPlay(1, 500);
                    break;
                }

                int rangeDataI = GetRangeADData();
                double rangeDataF = RangeADToConvert(rangeDataI);     //距離データを変換

                if (0 < rangeDataF && rangeDataF <= 30) //距離が３０cm以内の時
                {
                    Random rnd = new Random();
                    int randomNumber = rnd.Next(6);
                    int motionNum = 4 + (randomNumber % 2);//右か左の選択

                    for (int j = 0; j < (randomNumber+2)/2; j++)
                    {

                        MotionPlay(motionNum, 500);

                    }
                }

                else                            //そうでないときは前進
                {
                    MotionPlay(13, 4000);
                }


            }
        }

        #endregion

        #region マッピングボタンを押したとき

        private void button5_Click(object sender, EventArgs e)
        {

            robotXY = new PointF(0, 0);
            robotAngle = 0;             //初期は０にしておく

            MotionPlay(1, 1000);

            for (int i = 0; i < 1000; i++)         //ずっとループを続ける可能性があるので、１０００回くらいでとめてやる
            {    
                if (stopFlag == true)      //ストップボタンが押されたとき
                {
                    stopFlag = false;
                    MotionPlay(1, 500);
                    break;
                }



                int rangeDataI = GetRangeADData();
                double rangeDataF = RangeADToConvert(rangeDataI);     //距離データを変換

           
                if (0 < rangeDataF && rangeDataF <= 30)
                {
                    TurnLeft();
                    TurnLeft();
                }
                else                            //そうでないときは前進
                {
                    MoveForward();
                }


                //オートマップ保存の判定
                if (autoMapSaveFlag == true)
                {
                    autoMapSave();
                }

            }
        }

        #endregion

        #region ５方向取得ボタンを押したとき

        private void button6_Click(object sender, EventArgs e)
        {

            robotXY = new PointF(0, 0);
            robotAngle = 0;             //初期は０にしておく
            MotionPlay(1, 1000);

            for (int i = 0; i < 1000; i++)         //ずっとループを続ける可能性があるので、１０００回くらいでとめてやる
            {

                if (stopFlag == true)      //ストップボタンが押されたとき
                {
                    stopFlag = false;
                    MotionPlay(1, 500);
                    break;
                }

                RangeDataScan(5, 90);

                int rangeDataI = GetRangeADData();
                double rangeDataF = RangeADToConvert(rangeDataI);     //距離データを変換

                if (0 < rangeDataF && rangeDataF <= 30)
                {
                    TurnLeft();
                    TurnLeft();
                }
                else                            //そうでないときは前進
                {
                    MoveForward();
                }

                //オートマップ保存の判定
                if (autoMapSaveFlag == true)
                {
                    autoMapSave();
                }

            }
        }


        #endregion

        #region 11方向取得ボタンを押したとき

        private void button7_Click(object sender, EventArgs e)
        {
            robotXY = new PointF(0, 0);
            robotAngle = 0;             //初期は０にしておく
            MotionPlay(1, 1000);

            for (int i = 0; i < 1000; i++)         //ずっとループを続ける可能性があるので、１０００回くらいでとめてやる
            {

                if (stopFlag == true)      //ストップボタンが押されたとき
                {
                    stopFlag = false;
                    MotionPlay(1, 500);
                    break;
                }

                RangeDataScan(11, 90);

                int rangeDataI = GetRangeADData();
                double rangeDataF = RangeADToConvert(rangeDataI);     //距離データを変換


                if (0 < rangeDataF && rangeDataF <= 30)
                {
                    TurnLeft();
                    TurnLeft();
                }
                else                            //そうでないときは前進
                {
                    MoveForward();
                }

                //オートマップ保存の判定
                if (autoMapSaveFlag == true)
                {
                    autoMapSave();
                }


            }
        }

        #endregion

        #region 行動計画つきボタンを押したとき
        /// <summary>
        /// 行動計画つきボタンを押したときのイベント関数
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button8_Click(object sender, EventArgs e)
        {
            robotXY = new PointF(0, 0);
            robotAngle = 0;             //初期は０にしておく
            MotionPlay(1, 1000);

            //それぞれのパラメータを作成
            int scanNum = 11;                   //スキャンする数
            double underFootWidth = 18.0;       //足元の距離設定
            



            double[] rangeDataList = new double[scanNum];

            for (int i = 0; i < 1000; i++)         //ずっとループを続ける可能性があるので、１０００回くらいでとめてやる
            {

                if (stopFlag == true)      //ストップボタンが押されたとき
                {
                    stopFlag = false;
                    MotionPlay(1, 500);
                    break;
                }


                ///
                ///　距離データを取得
                ///
                RangeDataScan2(rangeDataList, 90);


                double rangeDataCenter = rangeDataList[(scanNum / 2) - 1];       //中央のデータを入れておく 

                ///
                ///  ＬとＲで一番近い数を入れる変数を用意
                ///
                double nearLData = 1000;        //とりあえずありえない数を入れておく
                double nearRData = 1000;        //

                double rangeLSum = 0;
                double rangeRSum = 0;


                //右左それぞれの距離の合計と最小値を計算

                for (int j = 0; j < (scanNum / 2); j++)
                {
                    int r = j + (scanNum / 2) + 1;  //右の距離データの入っている配列の番号

                    //rangeLSum += rangeDataList[j];
                    //rangeRSum += rangeDataList[r];
                    //合計値を入れるが、負の数の場合は遠い場合なので、80cmを足す
                    rangeLSum += rangeDataList[j] > 0 ? rangeDataList[j] : 80;
                    rangeRSum += rangeDataList[r] > 0 ? rangeDataList[r] : 80;


                    ///
                    /// 小さい数を入れていく（ただし-1が返ってくる場合もあるので注意）
                    ///

                    if (rangeDataList[j] > 0 && rangeDataList[j] < nearLData)
                    {
                        nearLData = rangeDataList[j];
                    }
                    if (rangeDataList[r] > 0 && rangeDataList[r] < nearRData)
                    {
                        nearRData = rangeDataList[r];
                    }
                }


                if (nearLData < underFootWidth && nearRData < underFootWidth)//両足元に障害物があった場合
                {
                    //後ろを向く
                    for (int l = 0; l < 8; l++)
                    {
                        TurnLeft();
                    }
                }
                else if (nearLData < underFootWidth)        //左側に障害物があった場合
                {
                    TurnRight();　//右を向く
                }

                else if (nearRData < underFootWidth)        //右側に障害物があった場合
                {
                    TurnLeft();     //左を向く
                }

                else
                {
                    if (0 < rangeDataCenter && rangeDataCenter <= 30)   //正面に障害物があった場合                                               
                    {
                        ///
                        /// 乱数をもとにモーションを繰り返す
                        ///

                        Random rnd = new Random();
                        int randomNumber = rnd.Next(3);

                        for (int k = 0; k < randomNumber + 1; k++)
                        {
                            if (rangeRSum > rangeLSum)
                            {
                                TurnRight();
                            }
                            else
                            {

                                TurnLeft();
                            }
                        }
                    }

                    else   //障害物がないときは前進
                    {
                        MoveForward();
                    }

                    //オートマップ保存の判定
                    if (autoMapSaveFlag == true)
                    {
                        autoMapSave();
                    }
                }
            }

        }

        #endregion


        #region 緊急停止ボタンが押されたとき
      

        /// <summary>
        /// 緊急停止ボタンが押されたとおき
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button3_Click(object sender, EventArgs e)
        {
            stopFlag = true;
        }

        #endregion

        #region 画像保存ボタンが押されたとき
        /// <summary>
        /// 画像保存ボタンを押したとき
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void saveButton_Click(object sender, EventArgs e)
        {
            pictureBox1.Invalidate();

            if (pictureBox1.Image != null)
            {
                if (imageSaveDialog.ShowDialog() == DialogResult.OK)        //ダイアログボタンでＯＫが押されたら
                {
                    pictureBox1.Image.Save(imageSaveDialog.FileName, System.Drawing.Imaging.ImageFormat.Bmp);
                }
            }
            else
            {
                MessageBox.Show("画像が表示されていません");
            }
        }
        #endregion

        #region マップ自動保存用の関数

        bool autoMapSaveFlag = false;
        long mapNumber = 0;                 //マップのナンバー

        string fileName = "";
        string filePath = "";

        /// <summary>
        /// オートマップ保存ボタンが押されたとき
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void autoMapSaveButton_Click(object sender, EventArgs e)
        {
            if (autoMapSaveFlag == false)
            {
                //フォルダブラウザのダイアログを開く
                if (imageFolderBrowserDialog.ShowDialog() == DialogResult.OK)
                {
                    autoMapSaveFlag = true;
                    autoMapSaveButton.Text = "自動保存中";
                    autoMapSaveButton.ForeColor = Color.Red;
                    fileName = DateTime.Now.ToString("yyyyMMddHHmm");       //時間データをファイル名に
                    mapNumber = 0;
                    filePath = imageFolderBrowserDialog.SelectedPath;
                }

            }
            else
            {
                autoMapSaveFlag = false;
                autoMapSaveButton.Text = "自動マップ保存";
                autoMapSaveButton.ForeColor = Color.Black;
                
            }

        }

        private void autoMapSave()
        {
            if (pictureBox1.Image != null)
            {
                  //pictureBox1.Image.Save(imageSaveDialog.FileName);
                string fullPass = filePath + "\\" + fileName + "_" + mapNumber.ToString("000") + ".bmp";
                pictureBox1.Image.Save(fullPass, System.Drawing.Imaging.ImageFormat.Bmp);
                mapNumber++;        //マップナンバーを次の数に進めておく
            }
            else
            {
                MessageBox.Show("画像が表示されていません");
            }
        }

        #endregion



    }
}
